#!/system/bin/sh
#
# Recovery Mod checker for HUAWEI Ascend P6
# by @surdu_petru RC on XDA !
#
###########################################################################


# Constant definition
LOGPATH="/cache/chargemon"
LOGFILE="${LOGPATH}/chargemon.log"
WORKDIR="${LOGPATH}"
 
# Busybox setup
if [ -x "/system/xbin/busybox" ]; then
  BUSYBOX="/system/xbin/busybox"
elif [ -x "/system/bin/busybox" ]; then
  BUSYBOX="/system/bin/busybox"
else
  BUSYBOX=""
fi

# Command setup
DATE="${BUSYBOX} date"
MKDIR="${BUSYBOX} mkdir"
CHOWN="${BUSYBOX} chown"
CHMOD="${BUSYBOX} chmod"
MV="${BUSYBOX} mv"
TOUCH="${BUSYBOX} touch"
CAT="${BUSYBOX} cat"
SLEEP="${BUSYBOX} sleep"
KILL="${BUSYBOX} kill"
RM="${BUSYBOX} rm"
PS="${BUSYBOX} ps"
GREP="${BUSYBOX} grep"
AWK="${BUSYBOX} awk"
EXPR="${BUSYBOX} expr"
MOUNT="${BUSYBOX} mount"
UMOUNT="${BUSYBOX} umount"
TAR="${BUSYBOX} tar"
CHROOT="${BUSYBOX} chroot"
HEXDUMP="${BUSYBOX} hexdump"
CP="${BUSYBOX} cp"

# Ascend P6 setup

BOOTREC_EVENT="/dev/input/event1"
BOOTREC_LED_RED="/sys/class/leds/red/brightness"
BOOTREC_LED_GREEN="/sys/class/leds/green/brightness"
BOOTREC_LED_BLUE="/sys/class/leds/blue/brightness"
 
# Environment variable definition
PATH="/sbin:/system/xbin:/system/bin"; export PATH

# Function definition for logging
ECHOL(){
  _DATETIME=`${DATE} +"%Y%m%d %H%M%S"`
  echo "${_DATETIME}: $*" >> ${LOGFILE}
  return 0
}
EXECL(){
  _DATETIME=`${DATE} +"%Y%m%d %H%M%S"`
  echo "${_DATETIME}: $*" >> ${LOGFILE}
  $* 2>> ${LOGFILE}
  _RET=$?
  echo "${_DATETIME}: RET=${_RET}" >> ${LOGFILE}
  return ${_RET}
}

# Logfile rotation
if [ ! -d "${LOGPATH}" ];then
  ${MKDIR} ${LOGPATH}
  ${CHOWN} system.cache ${LOGPATH}
  ${CHMOD} 755 ${LOGPATH}
else
  if [ -f ${LOGFILE} ];then
    ${MV} ${LOGFILE} ${LOGFILE}.old
    ${CHMOD} 644 ${LOGFILE}.old
  fi
  ${TOUCH} ${LOGFILE}
  ${CHMOD} 644 ${LOGFILE}
fi
 
# Start main routine
ECHOL "### chargemon start..."
ECHOL "### BUSYBOX=${BUSYBOX}"

# Make work directory
ECHOL "### check workdir..."
if [ ! -d "${WORKDIR}" ];then
  EXECL ${MKDIR} ${WORKDIR}
  EXECL ${CHOWN} system.cache ${WORKDIR}
  EXECL ${CHMOD} 770 ${WORKDIR}
fi
 
ECHOL "### keycheck..."
 
# Turn on red LED
echo 255 > ${BOOTREC_LED_RED}
echo 0 > ${BOOTREC_LED_GREEN}
echo 0 > ${BOOTREC_LED_BLUE}
 
${SLEEP} 1

# Turn off red LED
echo 0 > ${BOOTREC_LED_RED}
echo 0 > ${BOOTREC_LED_GREEN}
echo 0 > ${BOOTREC_LED_BLUE}

stop
${CAT} ${BOOTREC_EVENT} > ${WORKDIR}/keycheck &
${SLEEP} 3
 
EXECL ${KILL} -9 $!

	if [ -s ${WORKDIR}/keycheck ]; then
		ECHOL "Vol down or up was pressed -> create recovery boot mode..."
		# remount rootfs to rw.
		ECHOL "### remount rootfs to rw..."
		EXECL ${MOUNT} -o remount,rw rootfs /
		# remount system to rw.
		ECHOL "### remount system to rw..."
		EXECL ${MOUNT} -o remount,rw /system

		ECHOL "Copying busybox to /sbin"
		EXECL ${BUSYBOX} ${CP} ${BUSYBOX} /sbin/
		BUSYBOX="/sbin/busybox"

		ECHOL "Creating symlinks in /sbin to all functions of busybox."
		# Create a symlink for each of the supported commands
		for sym in `${BUSYBOX} --list`; do
		ECHOL "linking ${BUSYBOX} to /sbin/$sym"
		${BUSYBOX} ln -sf ${BUSYBOX} /sbin/$sym
		done

		EXECL ${TAR} -xf /system/bin/recovery.tar
               
                /sbin/recovery
        else
        	start
	fi



# Turn off LEDs
echo 0 > ${BOOTREC_LED_RED}
echo 0 > ${BOOTREC_LED_GREEN}
echo 0 > ${BOOTREC_LED_BLUE}

ECHOL "Not key pressed ..." 
ECHOL "... normal boot mode ..."
